 
import numpy as np

import  unittest

from speedyfit.speedyfit import model

class TestModel(unittest.TestCase):
   
   #def setUp(self):
      
      #self.evolution_model = 'mist'
      #self.variables = ['log_L', 'log_Teff', 'log_g', 'M_H']
      #self.limits = [(0.2, 1.1), (-1.0, 0.25), (5.0, 9.0)]
      #self.obs = np.array([-0.55, 3.67, 4.50, -0.35])
      #self.obs_err = np.array([0.15, 0.05, 0.50, 0.40])
      
   def test_itable_single(self):
      
      photbands = ['STROMGREN.U', 'STROMGREN.B']
      gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/ikurucz93_z0.0_k2odfnew_sed_lawfitzpatrick2004_Rv3.10.fits'
      
      axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                                                                         teffrange=(4000, 5000), loggrange=(3.5, 4.0),
                                                                         ebvrange=(0.0, 0.02),
                                                                         variables=['teff','logg','ebv'])
      
      grid = [axis_values, pixelgrid]
      
      #-- check if interpolated model is correct
      flux1, Labs1 = model.get_itable_single(teff=4750, logg=3.75, ebv=0.01, grid=grid)
      
      self.assertAlmostEqual(Labs1, 0.45605564, 3,
                             msg='Basic model: Luminosity is not correct ({} != 0.456)'.format(Labs1))
      
      #-- check if radius is treated correctly
      flux1, Labs1 = model.get_itable_single(teff=4750, logg=3.75, ebv=0.01, rad=1, grid=grid)
      self.assertAlmostEqual(Labs1, 0.45605564, 3,
                             msg='Solar Radius: Luminosity is not correct ({} != 0.456)'.format(Labs1))
      
      flux1, Labs1 = model.get_itable_single(teff=4750, logg=3.75, ebv=0.01, rad=2, grid=grid)
      self.assertAlmostEqual(Labs1, 2**2*0.45605564, 3,
                             msg='Double Radius: Luminosity is not correct ({} != 1.824)'.format(Labs1))
      
      #-- compare with get_itable
      flux1, Labs1 = model.get_itable_single(teff=4750, logg=3.75, ebv=0.01, grid=grid)
      flux2, Labs2 = model.get_itable(teff=4750, logg=3.75, ebv=0.01, grid=grid)
      
      
      for f1, f2 in zip(flux1, flux2):
         self.assertEqual(f1, f2, 
                          msg="flux output of get_itable should be same as get_itable_single, ({} != {})".format(flux1, flux2))
      
      self.assertEqual(Labs1[0], Labs2['L'], 
                          msg="Labs output of get_itable should be same as get_itable_single, ({} != {})".format(Labs1[0], Labs2['L']))
      
      
   def test_itable(self):
      
      photbands = ['STROMGREN.U', 'STROMGREN.B']
      gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/ikurucz93_z0.0_k2odfnew_sed_lawfitzpatrick2004_Rv3.10.fits'
      
      
      axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                                                                         teffrange=(4000, 5000), loggrange=(3.5, 4.0),
                                                                         ebvrange=(0.0, 0.02),
                                                                         variables=['teff','logg','ebv'])
      
      grid1 = [axis_values, pixelgrid]
      
      gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/iTMAP2012_sdB_extended_lawfitzpatrick2004_Rv3.10.fits'
      
      axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                                                                         teffrange=(20000, 40000), loggrange=(5.0, 6.5),
                                                                         ebvrange=(0.0, 0.02),
                                                                         variables=['teff','logg','ebv'])
      
      grid2 = [axis_values, pixelgrid]
      
      flux1, Labs1 = model.get_itable(teff=4750, logg=3.75, ebv=0.01, rad=1, grid=grid1)
      flux2, Labs2 = model.get_itable(teff=4250, logg=3.50, ebv=0.01, rad=1, grid=grid1)
      
      fluxt, Labst = model.get_itable(teff=4750, logg=3.75, ebv=0.01, rad=1,
                                      teff2=4250, logg2=3.50, ebv2=0.01, rad2=1, grid=[grid1, grid1])
      
      for f1, f2, ft in zip(flux1, flux2, fluxt):
         self.assertEqual(f1+f2, ft, 
                          msg="binary SED should have sum of flux of single SED, ({} + {} != {})".format(flux1, flux2, fluxt))
      
      
      flux1, Labs1 = model.get_itable(teff=4800, logg=3.8, ebv=0.01, rad=2, grid=grid1)
      flux2, Labs2 = model.get_itable(teff=25000, logg=5.8, ebv=0.01, rad=0.14, grid=grid2)
      
      fluxt, Labst = model.get_itable(teff=4800, logg=3.8, ebv=0.01, rad=2,
                                      teff2=25000, logg2=5.8, ebv2=0.01, rad2=0.14,
                                      grid=[grid1, grid2])
      
      for f1, f2, ft in zip(flux1, flux2, fluxt):
         self.assertEqual(f1+f2, ft, 
                          msg="binary SED should have sum of flux of single SED, ({} + {} != {})".format(flux1, flux2, fluxt))
      
      
   def test_itable_vega(self):
      
      photbands = ['STROMGREN.U', 'STROMGREN.B', 'STROMGREN.V', 'STROMGREN.Y','2MASS.KS']
      gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/ikurucz93_z0.0_k2odfnew_sed_lawfitzpatrick2004_Rv3.10.fits'
      
      
      axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                                                                         teffrange=(8000, 10000), loggrange=(4.0, 5.0),
                                                                         ebvrange=(0.0, 0.02),
                                                                         variables=['teff','logg','ebv'])
      
      grid = [axis_values, pixelgrid]
      
      syn, Labs = model.get_itable_single(teff=9602, logg=4.10, ebv=0.0, rad=2.362, grid=grid)
      
      syn = syn / 340745610.888946**2 # correct for distance
      Labs = Labs[0]
      
      obs = np.array([8.528e-10, 5.664e-09, 6.190e-09, 3.547e-09, 3.803e-11])
      obs_e = np.array([3.458e-11, 1.484e-10, 1.788e-10, 9.148e-11, 6.515e-12])
      
      ratio = (obs/syn)
      weights = (obs/obs_e)
      scale = np.average(ratio,weights=weights)
      e_scale = np.sqrt(np.dot(weights, (ratio-scale)**2)/weights.sum())
      
      self.assertTrue(abs(scale - 1.0) < e_scale, 
                      msg="Cannot reproduce VEGA at correct distance! scale = {:0.1f}+-{:0.1f} instead of 1".format(scale, e_scale))
      
      self.assertTrue(abs(Labs - 40) < 2.5, 
                      msg="Vega should have luminosity of 40.12 Lsol, model gives {:0.2f} ".format(Labs))
      
   def test_itable_EG50(self):
      """
      Test the returned flux for a known white dwarf star EG 50
      Teff = 22290 +- 350 K
      logg = 8.10 +- 0.05
      Rad = 0.012 +- 0.001 Rsol
      d = 19 +- 3 pc
      """
      
      photbands = ['STROMGREN.U', 'STROMGREN.V', 'STROMGREN.B', 'STROMGREN.Y','2MASS.H']
      gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/iTMAP2012_sdB_extended_lawfitzpatrick2004_Rv3.10.fits'
      
      axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                                                                         teffrange=(20000, 40000), loggrange=(5.0, 6.5),
                                                                         ebvrange=(0.0, 0.02),
                                                                         variables=['teff','logg','ebv'])
      
      grid = [axis_values, pixelgrid]
      
      syn, Labs = model.get_itable_single(teff=22290, logg=6.5, ebv=0.0, rad=0.012, grid=grid)
      
      syn = syn / 842950390.9165244**2 # correct for distance
      Labs = Labs[0]
      
      obs = np.array([4.89e-14, 1.02e-13, 9.31e-14, 5.34e-14, 9.76e-16])
      obs_e = np.array([2.53e-15, 3.99e-15, 3.08e-15, 1.72e-15, 5.48e-17])
      
      ratio = (obs/syn)
      weights = (obs/obs_e)
      scale = np.average(ratio,weights=weights)
      e_scale = np.sqrt(np.dot(weights, (ratio-scale)**2)/weights.sum())
      
      self.assertTrue(abs(scale - 1.0) < e_scale, 
                      msg="Cannot reproduce EG 50 at correct distance! scale = {:0.1f}+-{:0.1f} instead of 1".format(scale, e_scale))
      


 
import numpy as np

import  unittest

from speedyfit.speedyfit import statfunc

class TestMCMC(unittest.TestCase):
      
   def test_derived_properties(self):
      
      theta = (26000, 5.8, 0.1429, 5771, 4.438, 1, 0.02)
      pnames = ('teff', 'logg', 'rad', 'teff2', 'logg2', 'rad2', 'ebv')
      
      derived_properties = statfunc.get_derived_properties(theta, pnames)
      
      self.assertAlmostEqual(derived_properties['mass'], 0.47, places=2, 
                           msg="sdB mass wrongly calculated: {}".format(derived_properties['mass']))
      
      self.assertAlmostEqual(derived_properties['mass2'], 1.00, places=2, 
                           msg="Solar mass wrongly calculated: {}".format(derived_properties['mass2']))
      
      self.assertAlmostEqual(derived_properties['q'], 0.47, places=2, 
                           msg="q wrongly calculated: {}".format(derived_properties['q']))
      
      
   def test_stat_chi2(self):
      
      meas = np.array([1.0,2.0,3.0])
      e_meas = np.array([0.25, 0.30, 0.15])
      syn = np.array([1.5, 1.89, 3.6])
      
      #-- colors (none scaled)
      colors = np.array([True,True,True])
      chi2, scale, e_scale = statfunc.stat_chi2(meas, e_meas, colors, syn)
      
      self.assertAlmostEqual(chi2, 20.1344, 2,
                             msg="Chi2 for colours not correct ({} != {})".format(chi2, 20.1344))
      
      #-- absolute magnitudes (scaled)
      colors = np.array([False,False,False])
      chi2, scale, e_scale = statfunc.stat_chi2(meas, e_meas, colors, syn)
      
      self.assertAlmostEqual(chi2, 3.32834, 2,
                             msg="Chi2 for colours not correct ({} != {})".format(chi2, 3.3283))
      
      #-- added distance
      constraints = {'distance': (1.5, 0.15)}
      colors = np.array([False,False,False])
      chi2, scale, e_scale = statfunc.stat_chi2(meas, e_meas, colors, syn, **constraints)
      
      self.assertAlmostEqual(chi2, 25.2343, 2,
                             msg="Chi2 for distance not correct ({} != {})".format(chi2, 25.2343))
      
      #self.assertTrue(False)


 
import numpy as np

import  unittest

from speedyfit.speedyfit import mcmc, statfunc


class TestMCMC(unittest.TestCase):
   
   #def setUp(self):
      
      #self.evolution_model = 'mist'
      #self.variables = ['log_L', 'log_Teff', 'log_g', 'M_H']
      #self.limits = [(0.2, 1.1), (-1.0, 0.25), (5.0, 9.0)]
      #self.obs = np.array([-0.55, 3.67, 4.50, -0.35])
      #self.obs_err = np.array([0.15, 0.05, 0.50, 0.40])
      
   
   def test_prior(self):
      
      theta = (26000, 5.8, 0.1429, 5771, 4.438, 1, 0.02)
      pnames = ('teff', 'logg', 'rad', 'teff2', 'logg2', 'rad2', 'ebv')
      
      limits = np.array([(20000, 40000), (5.0, 6.5), (0.05, 0.25), 
                         (4000, 8000),   (4.0, 5.0), (0.50, 2.10),
                         (0.00, 0.05)])
      
      derived_limits = {'q':(0.1, 1.0), 'mass':(0.40, 0.55), 'mass2':(0.70, 2.0)}
      
      derived_properties = statfunc.get_derived_properties(theta, pnames)
      
      prior = mcmc.lnprior(theta, derived_properties, limits,
                           pnames = pnames, derived_limits=derived_limits)
      
      self.assertEqual(prior, 0,
                       msg="Prior should have been 0 for accepted limits, was {}".format(prior))
      
      
      
      theta = (15000, 5.8, 0.1429, 5771, 4.438, 1, 0.02)
      derived_properties = statfunc.get_derived_properties(theta, pnames)
      
      prior = mcmc.lnprior(theta, derived_properties, limits,
                           pnames = pnames, derived_limits=derived_limits)
      
      self.assertEqual(prior, -np.inf,
                       msg="theta out of limits, expected prior = -inf, was {}".format(prior))
      
      
      
      theta = (26000, 5.8, 0.10, 5771, 4.438, 1, 0.02)
      derived_properties = statfunc.get_derived_properties(theta, pnames)
      
      prior = mcmc.lnprior(theta, derived_properties, limits,
                           pnames = pnames, derived_limits=derived_limits)
      
      self.assertEqual(prior, -np.inf,
                       msg="mass out of limits, expected prior = -inf, was {}".format(prior))
      
      
   #def test_likelihood(self):
      
      #photbands = ['STROMGREN.U', 'STROMGREN.B']
      #gridfilename = '/home/joris/Python/ivsdata/sedtables/modelgrids/ikurucz93_z0.0_k2odfnew_sed_lawfitzpatrick2004_Rv3.10.fits'
      
      #axis_values, grid_pars, pixelgrid, grid_names = model.prepare_grid(photbands, gridfilename,
                 #teffrange=(4000, 5000),loggrange=(3.5, 4.0),
                 #ebvrange=(0.0, 0.02),
                 #variables=['teff','logg','ebv'])
      
      #grids = [[axis_values, pixelgrid],]
      
      
      #theta = (4700, 3.8, 0.01)
      #pnames = ['teff', 'logg', 'ebv']
      #derived_properties = statfunc.get_derived_properties(theta, pnames)
      
      #y = np.array([5.0, 6.0])
      #yerr = np.array([0.5, 0.75])
      
      #kwargs = dict(pnames = pnames,
                    #photbands = photbands, 
                    #grid=grids)
      
      #print mcmc.lnlike(theta, derived_properties, y, yerr, **kwargs)
      
      
      #self.assertTrue(True)
      


# -*- coding: utf-8 -*-
import numpy as np

import warnings


def get_derived_properties(**pars):
   """
   Function that will derive several properties based on the chosen models
   
   Currently the following properties are calculated:
   - mass
   - mass1
   - mass2
   - q
   
   returns dictionary of all properties that could be calculated.
   """
   
   derived_properties = {}
   
   GG = 6.67384e-08
   Rsol = 69550800000.0
   Msol = 1.988547e+33
   
   #-- derive masses
   if 'rad' in pars and 'logg' in pars:
      mass = 10**pars['logg'] * (pars['rad'] * Rsol)**2 / GG
      derived_properties['mass'] = mass / Msol
   
   if 'rad' in pars and 'g' in pars:
      mass = pars['g'] * (pars['rad'] * Rsol)**2 / GG
      derived_properties['mass'] = mass / Msol
   
   if 'rad2' in pars and 'logg2' in pars:
      mass = 10**pars['logg2'] * (pars['rad2'] * Rsol)**2 / GG
      derived_properties['mass2'] = mass / Msol
      
   if 'rad2' in pars and 'g2' in pars:
      mass = pars['g2'] * (pars['rad2'] * Rsol)**2 / GG
      derived_properties['mass2'] = mass / Msol  
   
   #-- derive radii
   if 'mass' in pars and 'logg' in pars:
      rad = np.sqrt(GG * pars['mass'] * Msol / 10**pars['logg'])
      derived_properties['rad'] = rad / Rsol
      
   if 'mass' in pars and 'g' in pars:
      rad = np.sqrt(GG * pars['mass'] * Msol / pars['g'])
      derived_properties['rad'] = rad / Rsol
      
   if 'mass2' in pars and 'logg2' in pars:
      rad = np.sqrt(GG * pars['mass2'] * Msol / 10**pars['logg2'])
      derived_properties['rad2'] = rad / Rsol
      
   if 'mass2' in pars and 'g2' in pars:
      rad = np.sqrt(GG * pars['mass2'] * Msol / pars['g2'])
      derived_properties['rad2'] = rad / Rsol
   
   #-- derive mass ratio
   if 'mass' in derived_properties and 'mass2' in derived_properties:
      derived_properties['q'] = derived_properties['mass'] / derived_properties['mass2']
      
   if 'mass' in pars and 'mass2' in pars:
      derived_properties['q'] = pars['mass'] / pars['mass2']
      
   #-- derive luminosity ratio
   if 'rad' in pars and 'teff' in pars and 'rad2' in pars and 'teff2' in pars:
      l1 = ( pars['rad']**2 * pars['teff']**4 )
      l2 = ( pars['rad2']**2 * pars['teff2']**4 )
      derived_properties['lr'] = l1 / l2
      
   if 'mass' in pars and 'teff' in pars and 'mass2' in pars and 'teff2' in pars:
      l1 = ( derived_properties['rad']**2 * pars['teff']**4 )
      l2 = ( derived_properties['rad2']**2 * pars['teff2']**4 )
      derived_properties['lr'] = l1 / l2
   
   #-- derive radius ratio
   if 'rad' in pars and 'rad2' in pars:
      derived_properties['rr'] = pars['rad'] / pars['rad2']
      
   if 'rad' in derived_properties and 'rad2' in derived_properties:
      derived_properties['rr'] = derived_properties['rad'] / derived_properties['rad2']
   
   #-- add empty values for luminosity and distance to prevent problems with 
   #   failed models
   derived_properties.update({'d':0, 'L':0})
   if 'rad2' in pars:
      derived_properties['L2'] = 0
   if hasattr(pars['teff'], '__iter__'):
      derived_properties['d'] = np.zeros_like(pars['teff'])
      derived_properties['L'] = np.zeros_like(pars['teff'])
      if 'rad2' in pars:
         derived_properties['L2'] = np.zeros_like(pars['teff'])
      
      
   return derived_properties

#def get_derived_properties(theta, pnames):
   #"""
   #Function that will derive several properties based on the chosen models
   
   #Currently the following properties are calculated:
   #- mass
   #- mass1
   #- mass2
   #- q
   
   #returns dictionary of all properties that could be calculated.
   #"""
   
   #derived_properties = {}
   
   #GG = 6.67384e-08
   #Rsol = 69550800000.0
   #Msol = 1.988547e+33
   
   ##-- derive masses
   #if 'rad' in pnames and 'logg' in pnames:
      #mass = 10**theta[pnames.index('logg')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
      #derived_properties['mass'] = mass / Msol
   
   #if 'rad' in pnames and 'g' in pnames:
      #mass = theta[pnames.index('g')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
      #derived_properties['mass'] = mass / Msol
   
   #if 'rad2' in pnames and 'logg2' in pnames:
      #mass = 10**theta[pnames.index('logg2')] * (theta[pnames.index('rad2')] * Rsol)**2 / GG
      #derived_properties['mass2'] = mass / Msol
      
   #if 'rad2' in pnames and 'g2' in pnames:
      #mass = theta[pnames.index('g2')] * (theta[pnames.index('rad2')] * Rsol)**2 / GG
      #derived_properties['mass2'] = mass / Msol  
   
   ##-- derive radii
   #if 'mass' in pnames and 'logg' in pnames:
      #rad = np.sqrt(GG * theta[pnames.index('mass')] * Msol / 10**theta[pnames.index('logg')])
      #derived_properties['rad'] = rad / Rsol
      
   #if 'mass' in pnames and 'g' in pnames:
      #rad = np.sqrt(GG * theta[pnames.index('mass')] * Msol / theta[pnames.index('g')])
      #derived_properties['rad'] = rad / Rsol
      
   #if 'mass2' in pnames and 'logg2' in pnames:
      #rad = np.sqrt(GG * theta[pnames.index('mass2')] * Msol / 10**theta[pnames.index('logg2')])
      #derived_properties['rad2'] = rad / Rsol
      
   #if 'mass2' in pnames and 'g2' in pnames:
      #rad = np.sqrt(GG * theta[pnames.index('mass2')] * Msol / theta[pnames.index('g2')])
      #derived_properties['rad2'] = rad / Rsol
   
   ##-- derive mass ratio
   #if 'mass' in derived_properties and 'mass2' in derived_properties:
      #derived_properties['q'] = derived_properties['mass'] / derived_properties['mass2']
      
   #if 'mass' in pnames and 'mass2' in pnames:
      #derived_properties['q'] = theta[pnames.index('mass')] / theta[pnames.index('mass2')]
      
   ##-- derive luminosity ratio
   #if 'rad' in pnames and 'teff' in pnames and 'rad2' in pnames and 'teff2' in pnames:
      #l1 = ( theta[pnames.index('rad')]**2 * theta[pnames.index('teff')]**4 )
      #l2 = ( theta[pnames.index('rad2')]**2 * theta[pnames.index('teff2')]**4 )
      #derived_properties['lr'] = l1 / l2
      
   #if 'mass' in pnames and 'teff' in pnames and 'mass2' in pnames and 'teff2' in pnames:
      #l1 = ( derived_properties['rad']**2 * theta[pnames.index('teff')]**4 )
      #l2 = ( derived_properties['rad2']**2 * theta[pnames.index('teff2')]**4 )
      #derived_properties['lr'] = l1 / l2
   
   ##-- add empty values for luminosity and distance to prevent problems with 
   ##   failed models
   #derived_properties.update({'d':0, 'L':0})
   #if 'rad2' in pnames:
      #derived_properties['L2'] = 0
   
   #return derived_properties

def get_derived_properties_binary(theta, pnames):
   """
   Function that will derive several properties based on the chosen models
   
   Currently the following properties are calculated:
   - mass
   - mass1
   - mass2
   - q
   
   returns dictionary of all properties that could be calculated.
   """
   
   derived_properties = {}
   
   GG = 6.67384e-08
   Rsol = 69550800000.0
   Msol = 1.988547e+33
   
   #-- derive masses
   if 'g' in pnames:
      m1 = theta[pnames.index('g')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
   else:   
      m1 = 10**theta[pnames.index('logg')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
   derived_properties['mass'] = m1 / Msol
   
   if 'g2' in pnames:
      m2 = theta[pnames.index('g2')] * (theta[pnames.index('rad2')] * Rsol)**2 / GG
   else:
      m2 = 10**theta[pnames.index('logg2')] * (theta[pnames.index('rad2')] * Rsol)**2 / GG
   derived_properties['mass2'] = m2 / Msol
   
   #-- derive mass ratio
   derived_properties['q'] = m1 / m2
   
   
   #-- derive luminosity ratio
   l1 = ( theta[pnames.index('rad')]**2 * theta[pnames.index('teff')]**4 )
   l2 = ( theta[pnames.index('rad2')]**2 * theta[pnames.index('teff2')]**4 )
   derived_properties['lr'] = l1 / l2
   
   #-- add empty values for luminosity and distance to prevent problems with 
   #   failed models
   derived_properties.update({'d':0, 'L':0, 'L2':0})
   
   return derived_properties

def get_derived_properties_single(theta, pnames):
   """
   Function that will derive several properties based on the chosen models
   
   Currently the following properties are calculated:
   - mass
   - mass1
   - mass2
   - q
   
   returns dictionary of all properties that could be calculated.
   """
   
   derived_properties = {}
   
   GG = 6.67384e-08
   Rsol = 69550800000.0
   Msol = 1.988547e+33
   
   #-- derive masses
   if 'g' in pnames:
      mass = theta[pnames.index('g')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
   else:
      mass = 10**theta[pnames.index('logg')] * (theta[pnames.index('rad')] * Rsol)**2 / GG
   derived_properties['mass'] = mass / Msol
   
   #-- add empty values for luminosity and distance to prevent problems with 
   #   failed models
   derived_properties.update({'d':0, 'L':0})
   
   return derived_properties

def stat_chi2(meas, e_meas, colors, syn, pars, **kwargs):
   """
   Calculate Chi2 and compute angular diameter.
   
   Colors and absolute fluxes are used to compute the Chi2, only absolute
   fluxes are used to compute angular diameter. If no absolute fluxes are
   given, the angular diameter is set to 0.
   
   If constraints are given in the kwargs they are included in the Chi2
   calculation. Accepted constrains are distance and q (mass ratio). Both
   should be given as a (value, error) tuple.
   
   @param meas: array of measurements
   @type meas: 1D array
   @param e_meas: array containing measurements errors
   @type e_meas: 1D array
   @param colors: boolean array separating colors (True) from absolute fluxes (False)
   @type colors: 1D boolean array
   @param syn: synthetic fluxes and colors
   @type syn: 1D array
   @param full_output: set to True if you want individual chisq
   @type full_output: boolean
   @return: chi-square, scale, e_scale
   @rtype: float,float,float
   """

   # check for NaNs in the observations
   nani = np.isnan(meas) | np.isnan(e_meas)
   if any(nani):
      warnings.warn('{} of the observed magnitudes are NaN values! (NaN values are ignored)'.format(np.sum(nani)))
   meas, e_meas, colors, syn = meas[~nani], e_meas[~nani], colors[~nani], syn[~nani]

   # First deal with Chi2 of the observations
   #=========================================
   #-- if syn represents only one measurement
   if sum(~colors) > 0:
      ratio = (meas/syn)[~colors]
      weights = (meas/e_meas)[~colors]
      #-- weighted average and standard deviation
      scale = np.average(ratio,weights=weights)
      e_scale = np.sqrt(np.dot(weights, (ratio-scale)**2)/weights.sum())
   else:
      scale,e_scale = 0,0
   #-- we don't need to scale the colors, only the absolute fluxes
   chisq = np.where(colors, (syn-meas)**2/e_meas**2, (syn*scale-meas)**2/e_meas**2)
   
   
   # Then add Chi2 of derived properties as distance, mass ratio, ...
   #=================================================================
   derived_properties = kwargs.get('derived_properties', {})
   constraints = kwargs.get('constraints', {})
   
   for con, (c, c_m, c_p) in list(constraints.items()):
      syn_c = None
      if con == 'distance' and sum(~colors) > 0:
         # distance can only be constrained if there is at least 1 absolute measurement
         c, c_m, c_p = 1./c**2, 2.*c_m/c**3, 2.*c_p/c**3
         syn_c = scale
         
      elif con in pars:
         #constrained on a fitted parameter
         syn_c = pars[con]
         
      elif con in derived_properties:
         # constrained on a derived parameter
         syn_c = derived_properties[con]
      
      if not syn_c is None:
         chi2_c = np.where(syn_c < c, (syn_c - c)**2 / c_m**2, (syn_c - c)**2 / c_p**2)
         
         # append to chisq array
         chisq = np.append(chisq, chi2_c)

   return chisq.sum(axis=0), scale, e_scale
  

def stat_chi2_multiple(meas, e_meas, colors, syn, pars, **kwargs):
   """
   Calculate Chi2 and compute angular diameter.
   
   Colors and absolute fluxes are used to compute the Chi2, only absolute
   fluxes are used to compute angular diameter. If no absolute fluxes are
   given, the angular diameter is set to 0.
   
   If constraints are given in the kwargs they are included in the Chi2
   calculation. Accepted constrains are distance and q (mass ratio). Both
   should be given as a (value, error) tuple.
   
   @param meas: array of measurements
   @type meas: 1D array
   @param e_meas: array containing measurements errors
   @type e_meas: 1D array
   @param colors: boolean array separating colors (True) from absolute fluxes (False)
   @type colors: 1D boolean array
   @param syn: synthetic fluxes and colors
   @type syn: 1D array
   @param full_output: set to True if you want individual chisq
   @type full_output: boolean
   @return: chi-square, scale, e_scale
   @rtype: float,float,float
   """
   
   # First deal with Chi2 of the observations
   #=========================================
   #-- if syn represents a grid of measurements
   if sum(~colors) > 0:
      ratio = (meas/syn)[~colors]
      weights = (meas/e_meas)[~colors]
      #-- weighted average and standard deviation
      scale = np.average(ratio,weights=weights.reshape(-1),axis=0)
      e_scale = np.sqrt(np.dot(weights.T, (ratio-scale)**2)/weights.sum(axis=0))[0]
   else:
      scale, e_scale = np.zeros(syn.shape[1]), np.zeros(syn.shape[1])
   
   #-- we don't need to scale the colors, only the absolute fluxes
   chisq = np.where(colors.reshape(-1,1), (syn-meas)**2/e_meas**2, (syn*scale-meas)**2/e_meas**2)
   
   
   # Then add Chi2 of derived properties as distance, mass ratio, ...
   #=================================================================
   derived_properties = kwargs.get('derived_properties', {})
   constraints = kwargs.get('constraints', {})
   
   for con, (c, c_m, c_p) in list(constraints.items()):
      syn_c = None
      if con == 'distance' and sum(~colors) > 0:
         # distance can only be constrained if there is at least 1 absolute measurement
         c, c_m, c_p = 1./c**2, 2.*c_m/c**3, 2.*c_p/c**3
         syn_c = scale
         
      elif con in pars:
         #constrained on a fitted parameter
         syn_c = pars[con]
         
      elif con in derived_properties:
         # constrained on a derived parameter
         syn_c = derived_properties[con]
      
      if not syn_c is None:
         chi2_c = np.where(syn_c < c, (syn_c - c)**2 / c_m**2, (syn_c - c)**2 / c_p**2)
         
         # append to chisq array
         chisq =  np.vstack([chisq, chi2_c])
   
   return chisq.sum(axis=0), scale, e_scale
